#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


        #include "build/csleigh.i"
        

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // char const *()(void *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 66), // void *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // char const *()(void *, void *, uint64_t, int32_t)
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  6 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24), // uint64_t
/*  7 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 21), // int32_t
/*  8 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION, 49), // csleigh_RegisterDefinition *()(void *, unsigned int *)
/* 10 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 11 */ _CFFI_OP(_CFFI_OP_POINTER, 16), // unsigned int *
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION, 31), // csleigh_TranslationResult *()(void *, unsigned char const *, unsigned int, uint64_t, unsigned int, _Bool)
/* 14 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 15 */ _CFFI_OP(_CFFI_OP_POINTER, 65), // unsigned char const *
/* 16 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 17 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24),
/* 18 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 19 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1), // _Bool
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION, 61), // int()(csleigh_Address *)
/* 22 */ _CFFI_OP(_CFFI_OP_POINTER, 42), // csleigh_Address *
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // void *()(char const *)
/* 25 */ _CFFI_OP(_CFFI_OP_POINTER, 41), // char const *
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // void *()(csleigh_Address *)
/* 28 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 29 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // void()(csleigh_TranslationResult *)
/* 31 */ _CFFI_OP(_CFFI_OP_POINTER, 54), // csleigh_TranslationResult *
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // void()(void *)
/* 34 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // void()(void *, char const *, uint32_t)
/* 37 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 39 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22), // uint32_t
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 41 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 42 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // csleigh_Address
/* 43 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // csleigh_BadDataError
/* 44 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // csleigh_Error
/* 45 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // csleigh_ErrorType
/* 46 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // csleigh_OpCode
/* 47 */ _CFFI_OP(_CFFI_OP_POINTER, 48), // csleigh_PcodeOp *
/* 48 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // csleigh_PcodeOp
/* 49 */ _CFFI_OP(_CFFI_OP_POINTER, 50), // csleigh_RegisterDefinition *
/* 50 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // csleigh_RegisterDefinition
/* 51 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // csleigh_SeqNum
/* 52 */ _CFFI_OP(_CFFI_OP_POINTER, 53), // csleigh_Translation *
/* 53 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // csleigh_Translation
/* 54 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 7), // csleigh_TranslationResult
/* 55 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 8), // csleigh_UnimplError
/* 56 */ _CFFI_OP(_CFFI_OP_POINTER, 57), // csleigh_Varnode *
/* 57 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 9), // csleigh_Varnode
/* 58 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 19), // int16_t
/* 59 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 23), // int64_t
/* 60 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 17), // int8_t
/* 61 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 62 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 20), // uint16_t
/* 63 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 18), // uint8_t
/* 64 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 26), // uintptr_t
/* 65 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 66 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_Address(csleigh_Address *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { void * *tmp = &p->space; (void)tmp; }
  (void)((p->offset) | 0);  /* check that 'csleigh_Address.offset' is an integer */
}
struct _cffi_align_typedef_csleigh_Address { char x; csleigh_Address y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_BadDataError(csleigh_BadDataError *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_Address *tmp = &p->address; (void)tmp; }
}
struct _cffi_align_typedef_csleigh_BadDataError { char x; csleigh_BadDataError y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_Error(csleigh_Error *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_ErrorType *tmp = &p->type; (void)tmp; }
  { char const * *tmp = &p->explain; (void)tmp; }
  { csleigh_UnimplError *tmp = &p->unimpl; (void)tmp; }
  { csleigh_BadDataError *tmp = &p->baddata; (void)tmp; }
}
struct _cffi_align_typedef_csleigh_Error { char x; csleigh_Error y; };

static int _cffi_const_csleigh_ERROR_TYPE_NOERROR(unsigned long long *o)
{
  int n = (csleigh_ERROR_TYPE_NOERROR) <= 0;
  *o = (unsigned long long)((csleigh_ERROR_TYPE_NOERROR) | 0);  /* check that csleigh_ERROR_TYPE_NOERROR is an integer */
  return n;
}

static int _cffi_const_csleigh_ERROR_TYPE_GENERIC(unsigned long long *o)
{
  int n = (csleigh_ERROR_TYPE_GENERIC) <= 0;
  *o = (unsigned long long)((csleigh_ERROR_TYPE_GENERIC) | 0);  /* check that csleigh_ERROR_TYPE_GENERIC is an integer */
  return n;
}

static int _cffi_const_csleigh_ERROR_TYPE_UNIMPL(unsigned long long *o)
{
  int n = (csleigh_ERROR_TYPE_UNIMPL) <= 0;
  *o = (unsigned long long)((csleigh_ERROR_TYPE_UNIMPL) | 0);  /* check that csleigh_ERROR_TYPE_UNIMPL is an integer */
  return n;
}

static int _cffi_const_csleigh_ERROR_TYPE_BADDATA(unsigned long long *o)
{
  int n = (csleigh_ERROR_TYPE_BADDATA) <= 0;
  *o = (unsigned long long)((csleigh_ERROR_TYPE_BADDATA) | 0);  /* check that csleigh_ERROR_TYPE_BADDATA is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_COPY(unsigned long long *o)
{
  int n = (csleigh_CPUI_COPY) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_COPY) | 0);  /* check that csleigh_CPUI_COPY is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_LOAD(unsigned long long *o)
{
  int n = (csleigh_CPUI_LOAD) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_LOAD) | 0);  /* check that csleigh_CPUI_LOAD is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_STORE(unsigned long long *o)
{
  int n = (csleigh_CPUI_STORE) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_STORE) | 0);  /* check that csleigh_CPUI_STORE is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_BRANCH(unsigned long long *o)
{
  int n = (csleigh_CPUI_BRANCH) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_BRANCH) | 0);  /* check that csleigh_CPUI_BRANCH is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_CBRANCH(unsigned long long *o)
{
  int n = (csleigh_CPUI_CBRANCH) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_CBRANCH) | 0);  /* check that csleigh_CPUI_CBRANCH is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_BRANCHIND(unsigned long long *o)
{
  int n = (csleigh_CPUI_BRANCHIND) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_BRANCHIND) | 0);  /* check that csleigh_CPUI_BRANCHIND is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_CALL(unsigned long long *o)
{
  int n = (csleigh_CPUI_CALL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_CALL) | 0);  /* check that csleigh_CPUI_CALL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_CALLIND(unsigned long long *o)
{
  int n = (csleigh_CPUI_CALLIND) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_CALLIND) | 0);  /* check that csleigh_CPUI_CALLIND is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_CALLOTHER(unsigned long long *o)
{
  int n = (csleigh_CPUI_CALLOTHER) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_CALLOTHER) | 0);  /* check that csleigh_CPUI_CALLOTHER is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_RETURN(unsigned long long *o)
{
  int n = (csleigh_CPUI_RETURN) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_RETURN) | 0);  /* check that csleigh_CPUI_RETURN is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_EQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_EQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_EQUAL) | 0);  /* check that csleigh_CPUI_INT_EQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_NOTEQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_NOTEQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_NOTEQUAL) | 0);  /* check that csleigh_CPUI_INT_NOTEQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SLESS(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SLESS) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SLESS) | 0);  /* check that csleigh_CPUI_INT_SLESS is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SLESSEQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SLESSEQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SLESSEQUAL) | 0);  /* check that csleigh_CPUI_INT_SLESSEQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_LESS(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_LESS) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_LESS) | 0);  /* check that csleigh_CPUI_INT_LESS is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_LESSEQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_LESSEQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_LESSEQUAL) | 0);  /* check that csleigh_CPUI_INT_LESSEQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_ZEXT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_ZEXT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_ZEXT) | 0);  /* check that csleigh_CPUI_INT_ZEXT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SEXT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SEXT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SEXT) | 0);  /* check that csleigh_CPUI_INT_SEXT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_ADD(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_ADD) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_ADD) | 0);  /* check that csleigh_CPUI_INT_ADD is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SUB(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SUB) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SUB) | 0);  /* check that csleigh_CPUI_INT_SUB is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_CARRY(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_CARRY) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_CARRY) | 0);  /* check that csleigh_CPUI_INT_CARRY is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SCARRY(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SCARRY) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SCARRY) | 0);  /* check that csleigh_CPUI_INT_SCARRY is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SBORROW(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SBORROW) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SBORROW) | 0);  /* check that csleigh_CPUI_INT_SBORROW is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_2COMP(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_2COMP) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_2COMP) | 0);  /* check that csleigh_CPUI_INT_2COMP is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_NEGATE(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_NEGATE) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_NEGATE) | 0);  /* check that csleigh_CPUI_INT_NEGATE is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_XOR(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_XOR) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_XOR) | 0);  /* check that csleigh_CPUI_INT_XOR is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_AND(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_AND) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_AND) | 0);  /* check that csleigh_CPUI_INT_AND is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_OR(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_OR) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_OR) | 0);  /* check that csleigh_CPUI_INT_OR is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_LEFT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_LEFT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_LEFT) | 0);  /* check that csleigh_CPUI_INT_LEFT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_RIGHT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_RIGHT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_RIGHT) | 0);  /* check that csleigh_CPUI_INT_RIGHT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SRIGHT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SRIGHT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SRIGHT) | 0);  /* check that csleigh_CPUI_INT_SRIGHT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_MULT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_MULT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_MULT) | 0);  /* check that csleigh_CPUI_INT_MULT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_DIV(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_DIV) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_DIV) | 0);  /* check that csleigh_CPUI_INT_DIV is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SDIV(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SDIV) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SDIV) | 0);  /* check that csleigh_CPUI_INT_SDIV is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_REM(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_REM) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_REM) | 0);  /* check that csleigh_CPUI_INT_REM is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INT_SREM(unsigned long long *o)
{
  int n = (csleigh_CPUI_INT_SREM) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INT_SREM) | 0);  /* check that csleigh_CPUI_INT_SREM is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_BOOL_NEGATE(unsigned long long *o)
{
  int n = (csleigh_CPUI_BOOL_NEGATE) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_BOOL_NEGATE) | 0);  /* check that csleigh_CPUI_BOOL_NEGATE is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_BOOL_XOR(unsigned long long *o)
{
  int n = (csleigh_CPUI_BOOL_XOR) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_BOOL_XOR) | 0);  /* check that csleigh_CPUI_BOOL_XOR is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_BOOL_AND(unsigned long long *o)
{
  int n = (csleigh_CPUI_BOOL_AND) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_BOOL_AND) | 0);  /* check that csleigh_CPUI_BOOL_AND is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_BOOL_OR(unsigned long long *o)
{
  int n = (csleigh_CPUI_BOOL_OR) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_BOOL_OR) | 0);  /* check that csleigh_CPUI_BOOL_OR is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_EQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_EQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_EQUAL) | 0);  /* check that csleigh_CPUI_FLOAT_EQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_NOTEQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_NOTEQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_NOTEQUAL) | 0);  /* check that csleigh_CPUI_FLOAT_NOTEQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_LESS(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_LESS) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_LESS) | 0);  /* check that csleigh_CPUI_FLOAT_LESS is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_LESSEQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_LESSEQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_LESSEQUAL) | 0);  /* check that csleigh_CPUI_FLOAT_LESSEQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_NAN(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_NAN) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_NAN) | 0);  /* check that csleigh_CPUI_FLOAT_NAN is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_ADD(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_ADD) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_ADD) | 0);  /* check that csleigh_CPUI_FLOAT_ADD is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_DIV(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_DIV) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_DIV) | 0);  /* check that csleigh_CPUI_FLOAT_DIV is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_MULT(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_MULT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_MULT) | 0);  /* check that csleigh_CPUI_FLOAT_MULT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_SUB(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_SUB) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_SUB) | 0);  /* check that csleigh_CPUI_FLOAT_SUB is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_NEG(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_NEG) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_NEG) | 0);  /* check that csleigh_CPUI_FLOAT_NEG is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_ABS(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_ABS) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_ABS) | 0);  /* check that csleigh_CPUI_FLOAT_ABS is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_SQRT(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_SQRT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_SQRT) | 0);  /* check that csleigh_CPUI_FLOAT_SQRT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_INT2FLOAT(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_INT2FLOAT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_INT2FLOAT) | 0);  /* check that csleigh_CPUI_FLOAT_INT2FLOAT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_FLOAT2FLOAT(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_FLOAT2FLOAT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_FLOAT2FLOAT) | 0);  /* check that csleigh_CPUI_FLOAT_FLOAT2FLOAT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_TRUNC(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_TRUNC) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_TRUNC) | 0);  /* check that csleigh_CPUI_FLOAT_TRUNC is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_CEIL(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_CEIL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_CEIL) | 0);  /* check that csleigh_CPUI_FLOAT_CEIL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_FLOOR(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_FLOOR) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_FLOOR) | 0);  /* check that csleigh_CPUI_FLOAT_FLOOR is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_FLOAT_ROUND(unsigned long long *o)
{
  int n = (csleigh_CPUI_FLOAT_ROUND) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_FLOAT_ROUND) | 0);  /* check that csleigh_CPUI_FLOAT_ROUND is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_MULTIEQUAL(unsigned long long *o)
{
  int n = (csleigh_CPUI_MULTIEQUAL) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_MULTIEQUAL) | 0);  /* check that csleigh_CPUI_MULTIEQUAL is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INDIRECT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INDIRECT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INDIRECT) | 0);  /* check that csleigh_CPUI_INDIRECT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_PIECE(unsigned long long *o)
{
  int n = (csleigh_CPUI_PIECE) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_PIECE) | 0);  /* check that csleigh_CPUI_PIECE is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_SUBPIECE(unsigned long long *o)
{
  int n = (csleigh_CPUI_SUBPIECE) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_SUBPIECE) | 0);  /* check that csleigh_CPUI_SUBPIECE is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_CAST(unsigned long long *o)
{
  int n = (csleigh_CPUI_CAST) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_CAST) | 0);  /* check that csleigh_CPUI_CAST is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_PTRADD(unsigned long long *o)
{
  int n = (csleigh_CPUI_PTRADD) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_PTRADD) | 0);  /* check that csleigh_CPUI_PTRADD is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_PTRSUB(unsigned long long *o)
{
  int n = (csleigh_CPUI_PTRSUB) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_PTRSUB) | 0);  /* check that csleigh_CPUI_PTRSUB is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_SEGMENTOP(unsigned long long *o)
{
  int n = (csleigh_CPUI_SEGMENTOP) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_SEGMENTOP) | 0);  /* check that csleigh_CPUI_SEGMENTOP is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_CPOOLREF(unsigned long long *o)
{
  int n = (csleigh_CPUI_CPOOLREF) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_CPOOLREF) | 0);  /* check that csleigh_CPUI_CPOOLREF is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_NEW(unsigned long long *o)
{
  int n = (csleigh_CPUI_NEW) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_NEW) | 0);  /* check that csleigh_CPUI_NEW is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_INSERT(unsigned long long *o)
{
  int n = (csleigh_CPUI_INSERT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_INSERT) | 0);  /* check that csleigh_CPUI_INSERT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_EXTRACT(unsigned long long *o)
{
  int n = (csleigh_CPUI_EXTRACT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_EXTRACT) | 0);  /* check that csleigh_CPUI_EXTRACT is an integer */
  return n;
}

static int _cffi_const_csleigh_CPUI_POPCOUNT(unsigned long long *o)
{
  int n = (csleigh_CPUI_POPCOUNT) <= 0;
  *o = (unsigned long long)((csleigh_CPUI_POPCOUNT) | 0);  /* check that csleigh_CPUI_POPCOUNT is an integer */
  return n;
}

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_PcodeOp(csleigh_PcodeOp *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_SeqNum *tmp = &p->seq; (void)tmp; }
  { csleigh_OpCode *tmp = &p->opcode; (void)tmp; }
  { csleigh_Varnode * *tmp = &p->output; (void)tmp; }
  { csleigh_Varnode * *tmp = &p->inputs; (void)tmp; }
  (void)((p->inputs_count) | 0);  /* check that 'csleigh_PcodeOp.inputs_count' is an integer */
}
struct _cffi_align_typedef_csleigh_PcodeOp { char x; csleigh_PcodeOp y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_RegisterDefinition(csleigh_RegisterDefinition *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char const * *tmp = &p->name; (void)tmp; }
  { csleigh_Varnode *tmp = &p->varnode; (void)tmp; }
}
struct _cffi_align_typedef_csleigh_RegisterDefinition { char x; csleigh_RegisterDefinition y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_SeqNum(csleigh_SeqNum *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_Address *tmp = &p->pc; (void)tmp; }
  (void)((p->uniq) | 0);  /* check that 'csleigh_SeqNum.uniq' is an integer */
  (void)((p->order) | 0);  /* check that 'csleigh_SeqNum.order' is an integer */
}
struct _cffi_align_typedef_csleigh_SeqNum { char x; csleigh_SeqNum y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_Translation(csleigh_Translation *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_Address *tmp = &p->address; (void)tmp; }
  (void)((p->length) | 0);  /* check that 'csleigh_Translation.length' is an integer */
  { char const * *tmp = &p->asm_mnem; (void)tmp; }
  { char const * *tmp = &p->asm_body; (void)tmp; }
  { csleigh_PcodeOp * *tmp = &p->ops; (void)tmp; }
  (void)((p->ops_count) | 0);  /* check that 'csleigh_Translation.ops_count' is an integer */
}
struct _cffi_align_typedef_csleigh_Translation { char x; csleigh_Translation y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_TranslationResult(csleigh_TranslationResult *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_Error *tmp = &p->error; (void)tmp; }
  { csleigh_Translation * *tmp = &p->instructions; (void)tmp; }
  (void)((p->instructions_count) | 0);  /* check that 'csleigh_TranslationResult.instructions_count' is an integer */
}
struct _cffi_align_typedef_csleigh_TranslationResult { char x; csleigh_TranslationResult y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_UnimplError(csleigh_UnimplError *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { csleigh_Address *tmp = &p->address; (void)tmp; }
  (void)((p->instruction_length) | 0);  /* check that 'csleigh_UnimplError.instruction_length' is an integer */
}
struct _cffi_align_typedef_csleigh_UnimplError { char x; csleigh_UnimplError y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_typedef_csleigh_Varnode(csleigh_Varnode *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { void * *tmp = &p->space; (void)tmp; }
  (void)((p->offset) | 0);  /* check that 'csleigh_Varnode.offset' is an integer */
  (void)((p->size) | 0);  /* check that 'csleigh_Varnode.size' is an integer */
}
struct _cffi_align_typedef_csleigh_Varnode { char x; csleigh_Varnode y; };

static char const * _cffi_d_csleigh_AddrSpace_getName(void * x0)
{
  return csleigh_AddrSpace_getName(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_AddrSpace_getName(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_AddrSpace_getName(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(25));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_AddrSpace_getName _cffi_d_csleigh_AddrSpace_getName
#endif

static void * _cffi_d_csleigh_Addr_getSpaceFromConst(csleigh_Address * x0)
{
  return csleigh_Addr_getSpaceFromConst(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_Addr_getSpaceFromConst(PyObject *self, PyObject *arg0)
{
  csleigh_Address * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (csleigh_Address *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_Addr_getSpaceFromConst(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_Addr_getSpaceFromConst _cffi_d_csleigh_Addr_getSpaceFromConst
#endif

static int _cffi_d_csleigh_Addr_isConstant(csleigh_Address * x0)
{
  return csleigh_Addr_isConstant(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_Addr_isConstant(PyObject *self, PyObject *arg0)
{
  csleigh_Address * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (csleigh_Address *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_Addr_isConstant(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_Addr_isConstant _cffi_d_csleigh_Addr_isConstant
#endif

static csleigh_RegisterDefinition * _cffi_d_csleigh_Sleigh_getAllRegisters(void * x0, unsigned int * x1)
{
  return csleigh_Sleigh_getAllRegisters(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_Sleigh_getAllRegisters(PyObject *self, PyObject *args)
{
  void * x0;
  unsigned int * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  csleigh_RegisterDefinition * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "csleigh_Sleigh_getAllRegisters", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_Sleigh_getAllRegisters(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(49));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_Sleigh_getAllRegisters _cffi_d_csleigh_Sleigh_getAllRegisters
#endif

static char const * _cffi_d_csleigh_Sleigh_getRegisterName(void * x0, void * x1, uint64_t x2, int32_t x3)
{
  return csleigh_Sleigh_getRegisterName(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_Sleigh_getRegisterName(PyObject *self, PyObject *args)
{
  void * x0;
  void * x1;
  uint64_t x2;
  int32_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "csleigh_Sleigh_getRegisterName", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint64_t);
  if (x2 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int32_t);
  if (x3 == (int32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_Sleigh_getRegisterName(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(25));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_Sleigh_getRegisterName _cffi_d_csleigh_Sleigh_getRegisterName
#endif

static void * _cffi_d_csleigh_createContext(char const * x0)
{
  return csleigh_createContext(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_createContext(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_createContext(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_createContext _cffi_d_csleigh_createContext
#endif

static void _cffi_d_csleigh_destroyContext(void * x0)
{
  csleigh_destroyContext(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_destroyContext(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { csleigh_destroyContext(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_csleigh_destroyContext _cffi_d_csleigh_destroyContext
#endif

static void _cffi_d_csleigh_freeResult(csleigh_TranslationResult * x0)
{
  csleigh_freeResult(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_freeResult(PyObject *self, PyObject *arg0)
{
  csleigh_TranslationResult * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (csleigh_TranslationResult *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { csleigh_freeResult(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_csleigh_freeResult _cffi_d_csleigh_freeResult
#endif

static void _cffi_d_csleigh_setVariableDefault(void * x0, char const * x1, uint32_t x2)
{
  csleigh_setVariableDefault(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_setVariableDefault(PyObject *self, PyObject *args)
{
  void * x0;
  char const * x1;
  uint32_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "csleigh_setVariableDefault", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { csleigh_setVariableDefault(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_csleigh_setVariableDefault _cffi_d_csleigh_setVariableDefault
#endif

static csleigh_TranslationResult * _cffi_d_csleigh_translate(void * x0, unsigned char const * x1, unsigned int x2, uint64_t x3, unsigned int x4, _Bool x5)
{
  return csleigh_translate(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_csleigh_translate(PyObject *self, PyObject *args)
{
  void * x0;
  unsigned char const * x1;
  unsigned int x2;
  uint64_t x3;
  unsigned int x4;
  _Bool x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  csleigh_TranslationResult * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "csleigh_translate", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(15), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(15), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, unsigned int);
  if (x2 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, uint64_t);
  if (x3 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, unsigned int);
  if (x4 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x5 = (_Bool)_cffi_to_c__Bool(arg5);
  if (x5 == (_Bool)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = csleigh_translate(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(31));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_csleigh_translate _cffi_d_csleigh_translate
#endif

static const struct _cffi_global_s _cffi_globals[] = {
  { "csleigh_AddrSpace_getName", (void *)_cffi_f_csleigh_AddrSpace_getName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_csleigh_AddrSpace_getName },
  { "csleigh_Addr_getSpaceFromConst", (void *)_cffi_f_csleigh_Addr_getSpaceFromConst, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 27), (void *)_cffi_d_csleigh_Addr_getSpaceFromConst },
  { "csleigh_Addr_isConstant", (void *)_cffi_f_csleigh_Addr_isConstant, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 21), (void *)_cffi_d_csleigh_Addr_isConstant },
  { "csleigh_CPUI_BOOL_AND", (void *)_cffi_const_csleigh_CPUI_BOOL_AND, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_BOOL_NEGATE", (void *)_cffi_const_csleigh_CPUI_BOOL_NEGATE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_BOOL_OR", (void *)_cffi_const_csleigh_CPUI_BOOL_OR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_BOOL_XOR", (void *)_cffi_const_csleigh_CPUI_BOOL_XOR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_BRANCH", (void *)_cffi_const_csleigh_CPUI_BRANCH, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_BRANCHIND", (void *)_cffi_const_csleigh_CPUI_BRANCHIND, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_CALL", (void *)_cffi_const_csleigh_CPUI_CALL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_CALLIND", (void *)_cffi_const_csleigh_CPUI_CALLIND, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_CALLOTHER", (void *)_cffi_const_csleigh_CPUI_CALLOTHER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_CAST", (void *)_cffi_const_csleigh_CPUI_CAST, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_CBRANCH", (void *)_cffi_const_csleigh_CPUI_CBRANCH, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_COPY", (void *)_cffi_const_csleigh_CPUI_COPY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_CPOOLREF", (void *)_cffi_const_csleigh_CPUI_CPOOLREF, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_EXTRACT", (void *)_cffi_const_csleigh_CPUI_EXTRACT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_ABS", (void *)_cffi_const_csleigh_CPUI_FLOAT_ABS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_ADD", (void *)_cffi_const_csleigh_CPUI_FLOAT_ADD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_CEIL", (void *)_cffi_const_csleigh_CPUI_FLOAT_CEIL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_DIV", (void *)_cffi_const_csleigh_CPUI_FLOAT_DIV, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_EQUAL", (void *)_cffi_const_csleigh_CPUI_FLOAT_EQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_FLOAT2FLOAT", (void *)_cffi_const_csleigh_CPUI_FLOAT_FLOAT2FLOAT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_FLOOR", (void *)_cffi_const_csleigh_CPUI_FLOAT_FLOOR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_INT2FLOAT", (void *)_cffi_const_csleigh_CPUI_FLOAT_INT2FLOAT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_LESS", (void *)_cffi_const_csleigh_CPUI_FLOAT_LESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_LESSEQUAL", (void *)_cffi_const_csleigh_CPUI_FLOAT_LESSEQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_MULT", (void *)_cffi_const_csleigh_CPUI_FLOAT_MULT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_NAN", (void *)_cffi_const_csleigh_CPUI_FLOAT_NAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_NEG", (void *)_cffi_const_csleigh_CPUI_FLOAT_NEG, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_NOTEQUAL", (void *)_cffi_const_csleigh_CPUI_FLOAT_NOTEQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_ROUND", (void *)_cffi_const_csleigh_CPUI_FLOAT_ROUND, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_SQRT", (void *)_cffi_const_csleigh_CPUI_FLOAT_SQRT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_SUB", (void *)_cffi_const_csleigh_CPUI_FLOAT_SUB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_FLOAT_TRUNC", (void *)_cffi_const_csleigh_CPUI_FLOAT_TRUNC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INDIRECT", (void *)_cffi_const_csleigh_CPUI_INDIRECT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INSERT", (void *)_cffi_const_csleigh_CPUI_INSERT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_2COMP", (void *)_cffi_const_csleigh_CPUI_INT_2COMP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_ADD", (void *)_cffi_const_csleigh_CPUI_INT_ADD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_AND", (void *)_cffi_const_csleigh_CPUI_INT_AND, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_CARRY", (void *)_cffi_const_csleigh_CPUI_INT_CARRY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_DIV", (void *)_cffi_const_csleigh_CPUI_INT_DIV, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_EQUAL", (void *)_cffi_const_csleigh_CPUI_INT_EQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_LEFT", (void *)_cffi_const_csleigh_CPUI_INT_LEFT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_LESS", (void *)_cffi_const_csleigh_CPUI_INT_LESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_LESSEQUAL", (void *)_cffi_const_csleigh_CPUI_INT_LESSEQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_MULT", (void *)_cffi_const_csleigh_CPUI_INT_MULT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_NEGATE", (void *)_cffi_const_csleigh_CPUI_INT_NEGATE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_NOTEQUAL", (void *)_cffi_const_csleigh_CPUI_INT_NOTEQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_OR", (void *)_cffi_const_csleigh_CPUI_INT_OR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_REM", (void *)_cffi_const_csleigh_CPUI_INT_REM, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_RIGHT", (void *)_cffi_const_csleigh_CPUI_INT_RIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SBORROW", (void *)_cffi_const_csleigh_CPUI_INT_SBORROW, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SCARRY", (void *)_cffi_const_csleigh_CPUI_INT_SCARRY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SDIV", (void *)_cffi_const_csleigh_CPUI_INT_SDIV, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SEXT", (void *)_cffi_const_csleigh_CPUI_INT_SEXT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SLESS", (void *)_cffi_const_csleigh_CPUI_INT_SLESS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SLESSEQUAL", (void *)_cffi_const_csleigh_CPUI_INT_SLESSEQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SREM", (void *)_cffi_const_csleigh_CPUI_INT_SREM, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SRIGHT", (void *)_cffi_const_csleigh_CPUI_INT_SRIGHT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_SUB", (void *)_cffi_const_csleigh_CPUI_INT_SUB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_XOR", (void *)_cffi_const_csleigh_CPUI_INT_XOR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_INT_ZEXT", (void *)_cffi_const_csleigh_CPUI_INT_ZEXT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_LOAD", (void *)_cffi_const_csleigh_CPUI_LOAD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_MULTIEQUAL", (void *)_cffi_const_csleigh_CPUI_MULTIEQUAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_NEW", (void *)_cffi_const_csleigh_CPUI_NEW, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_PIECE", (void *)_cffi_const_csleigh_CPUI_PIECE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_POPCOUNT", (void *)_cffi_const_csleigh_CPUI_POPCOUNT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_PTRADD", (void *)_cffi_const_csleigh_CPUI_PTRADD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_PTRSUB", (void *)_cffi_const_csleigh_CPUI_PTRSUB, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_RETURN", (void *)_cffi_const_csleigh_CPUI_RETURN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_SEGMENTOP", (void *)_cffi_const_csleigh_CPUI_SEGMENTOP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_STORE", (void *)_cffi_const_csleigh_CPUI_STORE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_CPUI_SUBPIECE", (void *)_cffi_const_csleigh_CPUI_SUBPIECE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_ERROR_TYPE_BADDATA", (void *)_cffi_const_csleigh_ERROR_TYPE_BADDATA, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_ERROR_TYPE_GENERIC", (void *)_cffi_const_csleigh_ERROR_TYPE_GENERIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_ERROR_TYPE_NOERROR", (void *)_cffi_const_csleigh_ERROR_TYPE_NOERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_ERROR_TYPE_UNIMPL", (void *)_cffi_const_csleigh_ERROR_TYPE_UNIMPL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "csleigh_Sleigh_getAllRegisters", (void *)_cffi_f_csleigh_Sleigh_getAllRegisters, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 9), (void *)_cffi_d_csleigh_Sleigh_getAllRegisters },
  { "csleigh_Sleigh_getRegisterName", (void *)_cffi_f_csleigh_Sleigh_getRegisterName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 3), (void *)_cffi_d_csleigh_Sleigh_getRegisterName },
  { "csleigh_createContext", (void *)_cffi_f_csleigh_createContext, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 24), (void *)_cffi_d_csleigh_createContext },
  { "csleigh_destroyContext", (void *)_cffi_f_csleigh_destroyContext, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 33), (void *)_cffi_d_csleigh_destroyContext },
  { "csleigh_freeResult", (void *)_cffi_f_csleigh_freeResult, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 30), (void *)_cffi_d_csleigh_freeResult },
  { "csleigh_setVariableDefault", (void *)_cffi_f_csleigh_setVariableDefault, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 36), (void *)_cffi_d_csleigh_setVariableDefault },
  { "csleigh_translate", (void *)_cffi_f_csleigh_translate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 13), (void *)_cffi_d_csleigh_translate },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "space", offsetof(csleigh_Address, space),
             sizeof(((csleigh_Address *)0)->space),
             _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "offset", offsetof(csleigh_Address, offset),
              sizeof(((csleigh_Address *)0)->offset),
              _CFFI_OP(_CFFI_OP_NOOP, 6) },
  { "address", offsetof(csleigh_BadDataError, address),
               sizeof(((csleigh_BadDataError *)0)->address),
               _CFFI_OP(_CFFI_OP_NOOP, 42) },
  { "type", offsetof(csleigh_Error, type),
            sizeof(((csleigh_Error *)0)->type),
            _CFFI_OP(_CFFI_OP_NOOP, 45) },
  { "explain", offsetof(csleigh_Error, explain),
               sizeof(((csleigh_Error *)0)->explain),
               _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "unimpl", offsetof(csleigh_Error, unimpl),
              sizeof(((csleigh_Error *)0)->unimpl),
              _CFFI_OP(_CFFI_OP_NOOP, 55) },
  { "baddata", offsetof(csleigh_Error, baddata),
               sizeof(((csleigh_Error *)0)->baddata),
               _CFFI_OP(_CFFI_OP_NOOP, 43) },
  { "seq", offsetof(csleigh_PcodeOp, seq),
           sizeof(((csleigh_PcodeOp *)0)->seq),
           _CFFI_OP(_CFFI_OP_NOOP, 51) },
  { "opcode", offsetof(csleigh_PcodeOp, opcode),
              sizeof(((csleigh_PcodeOp *)0)->opcode),
              _CFFI_OP(_CFFI_OP_NOOP, 46) },
  { "output", offsetof(csleigh_PcodeOp, output),
              sizeof(((csleigh_PcodeOp *)0)->output),
              _CFFI_OP(_CFFI_OP_NOOP, 56) },
  { "inputs", offsetof(csleigh_PcodeOp, inputs),
              sizeof(((csleigh_PcodeOp *)0)->inputs),
              _CFFI_OP(_CFFI_OP_NOOP, 56) },
  { "inputs_count", offsetof(csleigh_PcodeOp, inputs_count),
                    sizeof(((csleigh_PcodeOp *)0)->inputs_count),
                    _CFFI_OP(_CFFI_OP_NOOP, 16) },
  { "name", offsetof(csleigh_RegisterDefinition, name),
            sizeof(((csleigh_RegisterDefinition *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "varnode", offsetof(csleigh_RegisterDefinition, varnode),
               sizeof(((csleigh_RegisterDefinition *)0)->varnode),
               _CFFI_OP(_CFFI_OP_NOOP, 57) },
  { "pc", offsetof(csleigh_SeqNum, pc),
          sizeof(((csleigh_SeqNum *)0)->pc),
          _CFFI_OP(_CFFI_OP_NOOP, 42) },
  { "uniq", offsetof(csleigh_SeqNum, uniq),
            sizeof(((csleigh_SeqNum *)0)->uniq),
            _CFFI_OP(_CFFI_OP_NOOP, 39) },
  { "order", offsetof(csleigh_SeqNum, order),
             sizeof(((csleigh_SeqNum *)0)->order),
             _CFFI_OP(_CFFI_OP_NOOP, 39) },
  { "address", offsetof(csleigh_Translation, address),
               sizeof(((csleigh_Translation *)0)->address),
               _CFFI_OP(_CFFI_OP_NOOP, 42) },
  { "length", offsetof(csleigh_Translation, length),
              sizeof(((csleigh_Translation *)0)->length),
              _CFFI_OP(_CFFI_OP_NOOP, 7) },
  { "asm_mnem", offsetof(csleigh_Translation, asm_mnem),
                sizeof(((csleigh_Translation *)0)->asm_mnem),
                _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "asm_body", offsetof(csleigh_Translation, asm_body),
                sizeof(((csleigh_Translation *)0)->asm_body),
                _CFFI_OP(_CFFI_OP_NOOP, 25) },
  { "ops", offsetof(csleigh_Translation, ops),
           sizeof(((csleigh_Translation *)0)->ops),
           _CFFI_OP(_CFFI_OP_NOOP, 47) },
  { "ops_count", offsetof(csleigh_Translation, ops_count),
                 sizeof(((csleigh_Translation *)0)->ops_count),
                 _CFFI_OP(_CFFI_OP_NOOP, 16) },
  { "error", offsetof(csleigh_TranslationResult, error),
             sizeof(((csleigh_TranslationResult *)0)->error),
             _CFFI_OP(_CFFI_OP_NOOP, 44) },
  { "instructions", offsetof(csleigh_TranslationResult, instructions),
                    sizeof(((csleigh_TranslationResult *)0)->instructions),
                    _CFFI_OP(_CFFI_OP_NOOP, 52) },
  { "instructions_count", offsetof(csleigh_TranslationResult, instructions_count),
                          sizeof(((csleigh_TranslationResult *)0)->instructions_count),
                          _CFFI_OP(_CFFI_OP_NOOP, 16) },
  { "address", offsetof(csleigh_UnimplError, address),
               sizeof(((csleigh_UnimplError *)0)->address),
               _CFFI_OP(_CFFI_OP_NOOP, 42) },
  { "instruction_length", offsetof(csleigh_UnimplError, instruction_length),
                          sizeof(((csleigh_UnimplError *)0)->instruction_length),
                          _CFFI_OP(_CFFI_OP_NOOP, 7) },
  { "space", offsetof(csleigh_Varnode, space),
             sizeof(((csleigh_Varnode *)0)->space),
             _CFFI_OP(_CFFI_OP_NOOP, 1) },
  { "offset", offsetof(csleigh_Varnode, offset),
              sizeof(((csleigh_Varnode *)0)->offset),
              _CFFI_OP(_CFFI_OP_NOOP, 6) },
  { "size", offsetof(csleigh_Varnode, size),
            sizeof(((csleigh_Varnode *)0)->size),
            _CFFI_OP(_CFFI_OP_NOOP, 39) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$csleigh_Address", 42, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_Address), offsetof(struct _cffi_align_typedef_csleigh_Address, y), 0, 2 },
  { "$csleigh_BadDataError", 43, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_BadDataError), offsetof(struct _cffi_align_typedef_csleigh_BadDataError, y), 2, 1 },
  { "$csleigh_Error", 44, 0,
    sizeof(csleigh_Error), offsetof(struct _cffi_align_typedef_csleigh_Error, y), 3, 4 },
  { "$csleigh_PcodeOp", 48, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_PcodeOp), offsetof(struct _cffi_align_typedef_csleigh_PcodeOp, y), 7, 5 },
  { "$csleigh_RegisterDefinition", 50, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_RegisterDefinition), offsetof(struct _cffi_align_typedef_csleigh_RegisterDefinition, y), 12, 2 },
  { "$csleigh_SeqNum", 51, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_SeqNum), offsetof(struct _cffi_align_typedef_csleigh_SeqNum, y), 14, 3 },
  { "$csleigh_Translation", 53, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_Translation), offsetof(struct _cffi_align_typedef_csleigh_Translation, y), 17, 6 },
  { "$csleigh_TranslationResult", 54, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_TranslationResult), offsetof(struct _cffi_align_typedef_csleigh_TranslationResult, y), 23, 3 },
  { "$csleigh_UnimplError", 55, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_UnimplError), offsetof(struct _cffi_align_typedef_csleigh_UnimplError, y), 26, 2 },
  { "$csleigh_Varnode", 57, _CFFI_F_CHECK_FIELDS,
    sizeof(csleigh_Varnode), offsetof(struct _cffi_align_typedef_csleigh_Varnode, y), 28, 3 },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "$csleigh_ErrorType", 45, _cffi_prim_int(sizeof(csleigh_ErrorType), ((csleigh_ErrorType)-1) <= 0),
    "csleigh_ERROR_TYPE_NOERROR,csleigh_ERROR_TYPE_GENERIC,csleigh_ERROR_TYPE_UNIMPL,csleigh_ERROR_TYPE_BADDATA" },
  { "$csleigh_OpCode", 46, _cffi_prim_int(sizeof(csleigh_OpCode), ((csleigh_OpCode)-1) <= 0),
    "csleigh_CPUI_COPY,csleigh_CPUI_LOAD,csleigh_CPUI_STORE,csleigh_CPUI_BRANCH,csleigh_CPUI_CBRANCH,csleigh_CPUI_BRANCHIND,csleigh_CPUI_CALL,csleigh_CPUI_CALLIND,csleigh_CPUI_CALLOTHER,csleigh_CPUI_RETURN,csleigh_CPUI_INT_EQUAL,csleigh_CPUI_INT_NOTEQUAL,csleigh_CPUI_INT_SLESS,csleigh_CPUI_INT_SLESSEQUAL,csleigh_CPUI_INT_LESS,csleigh_CPUI_INT_LESSEQUAL,csleigh_CPUI_INT_ZEXT,csleigh_CPUI_INT_SEXT,csleigh_CPUI_INT_ADD,csleigh_CPUI_INT_SUB,csleigh_CPUI_INT_CARRY,csleigh_CPUI_INT_SCARRY,csleigh_CPUI_INT_SBORROW,csleigh_CPUI_INT_2COMP,csleigh_CPUI_INT_NEGATE,csleigh_CPUI_INT_XOR,csleigh_CPUI_INT_AND,csleigh_CPUI_INT_OR,csleigh_CPUI_INT_LEFT,csleigh_CPUI_INT_RIGHT,csleigh_CPUI_INT_SRIGHT,csleigh_CPUI_INT_MULT,csleigh_CPUI_INT_DIV,csleigh_CPUI_INT_SDIV,csleigh_CPUI_INT_REM,csleigh_CPUI_INT_SREM,csleigh_CPUI_BOOL_NEGATE,csleigh_CPUI_BOOL_XOR,csleigh_CPUI_BOOL_AND,csleigh_CPUI_BOOL_OR,csleigh_CPUI_FLOAT_EQUAL,csleigh_CPUI_FLOAT_NOTEQUAL,csleigh_CPUI_FLOAT_LESS,csleigh_CPUI_FLOAT_LESSEQUAL,csleigh_CPUI_FLOAT_NAN,csleigh_CPUI_FLOAT_ADD,csleigh_CPUI_FLOAT_DIV,csleigh_CPUI_FLOAT_MULT,csleigh_CPUI_FLOAT_SUB,csleigh_CPUI_FLOAT_NEG,csleigh_CPUI_FLOAT_ABS,csleigh_CPUI_FLOAT_SQRT,csleigh_CPUI_FLOAT_INT2FLOAT,csleigh_CPUI_FLOAT_FLOAT2FLOAT,csleigh_CPUI_FLOAT_TRUNC,csleigh_CPUI_FLOAT_CEIL,csleigh_CPUI_FLOAT_FLOOR,csleigh_CPUI_FLOAT_ROUND,csleigh_CPUI_MULTIEQUAL,csleigh_CPUI_INDIRECT,csleigh_CPUI_PIECE,csleigh_CPUI_SUBPIECE,csleigh_CPUI_CAST,csleigh_CPUI_PTRADD,csleigh_CPUI_PTRSUB,csleigh_CPUI_SEGMENTOP,csleigh_CPUI_CPOOLREF,csleigh_CPUI_NEW,csleigh_CPUI_INSERT,csleigh_CPUI_EXTRACT,csleigh_CPUI_POPCOUNT" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "csleigh_AddrSpace", 1 },
  { "csleigh_Address", 42 },
  { "csleigh_BadDataError", 43 },
  { "csleigh_Context", 1 },
  { "csleigh_Error", 44 },
  { "csleigh_ErrorType", 45 },
  { "csleigh_OpCode", 46 },
  { "csleigh_PcodeOp", 48 },
  { "csleigh_RegisterDefinition", 50 },
  { "csleigh_SeqNum", 51 },
  { "csleigh_Translation", 53 },
  { "csleigh_TranslationResult", 54 },
  { "csleigh_UnimplError", 55 },
  { "csleigh_Varnode", 57 },
  { "int1", 60 },
  { "int2", 58 },
  { "int4", 7 },
  { "int8", 59 },
  { "intb", 59 },
  { "intm", 7 },
  { "uint1", 63 },
  { "uint2", 62 },
  { "uint4", 39 },
  { "uint8", 6 },
  { "uintb", 6 },
  { "uintm", 39 },
  { "uintp", 64 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  85,  /* num_globals */
  10,  /* num_struct_unions */
  2,  /* num_enums */
  27,  /* num_typenames */
  NULL,  /* no includes */
  67,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__csleigh(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__csleigh(void) { return NULL; }
#  else
     init_csleigh(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__csleigh(void)
{
  return _cffi_init("pypcode._csleigh", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_csleigh(void)
{
  _cffi_init("pypcode._csleigh", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
